#ifndef ISL_ID_SET_H
#define ISL_ID_SET_H

#define ISL_HSET			isl_id_set
#define ISL_HSET_EL			isl_id
#define ISL_HSET_IS_EQUAL		isl_id_set_is_equal
#define ISL_HSET_HAVE_READ_FROM_STR

#include <isl/hset.h>

#undef ISL_HSET
#undef ISL_HSET_EL

#endif
